@@ -12,9 +12,12 @@
 #' @usage lhs \%>\% rhs
 NULL
 
-#' @importFrom mosaicCore makeFun df_stats
+#' @importFrom mosaicCore makeFun df_stats na.warn
 #' @export
 mosaicCore::makeFun
 
 #' @export
-mosaicCore::df_stats
\ No newline at end of file
+mosaicCore::df_stats
+
+#' @export
+mosaicCore::na.warn
